using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace HideTheGamepad
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class HideTheGamepadGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // Game World
        SpriteFont messageFont;
        Rectangle screenRectangle;

        Texture2D titleTexture;
        Texture2D instructionTexture;
        Texture2D gameTexture;
        Texture2D warningTexture;
        Texture2D playtexture;
        Texture2D gameWonTexture;
        Texture2D gameLostTexture;

        int countdownTimerMins;
        int countdownTimerSecs;
        int countdownTimerTicks;

        enum GameState
        {
            TitleScreen,
            InstructionScreen,
            GameActive,
            GameWon,
            GameLost
        }

        GameState state = GameState.TitleScreen;

        SoundEffect tick;
        SoundEffect loudTick;
        SoundEffect win;
        SoundEffect lose;

        public HideTheGamepadGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            setScreenSizeInfo();
            screenRectangle = new Rectangle(minDisplayX, minDisplayY, displayWidth, displayHeight);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            titleTexture = Content.Load<Texture2D>("Images\\TitleScreen");
            instructionTexture = Content.Load<Texture2D>("Images\\InstructionScreen");
            gameTexture = Content.Load<Texture2D>("Images\\GameScreen");
            warningTexture = Content.Load<Texture2D>("Images\\WarningScreen");
            gameWonTexture = Content.Load<Texture2D>("Images\\GameWonScreen");
            gameLostTexture = Content.Load<Texture2D>("Images\\GameLostScreen");
            messageFont = Content.Load<SpriteFont>("messageFont");

            lose = Content.Load<SoundEffect>("Sounds\\lose");
            loudTick = Content.Load<SoundEffect>("Sounds\\loudTick");
            tick = Content.Load<SoundEffect>("Sounds\\tick");
            win = Content.Load<SoundEffect>("Sounds\\win");

        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            GamePadState pad1 = GamePad.GetState(PlayerIndex.One);
            GamePadState pad2 = GamePad.GetState(PlayerIndex.Two);
            // pad2 = pad1; // allows testing with one controller

            // Allows the game to exit
            if (pad1.Buttons.Back == ButtonState.Pressed)
                this.Exit();

            switch (state)
            {
                case GameState.TitleScreen:
                    if (pad1.Buttons.Start == ButtonState.Pressed)
                    {
                        state = GameState.InstructionScreen;
                    }
                    break;
                case GameState.InstructionScreen:

                    if (Keyboard.GetState().IsKeyDown(Keys.Space))
                    {
                        countdownTimerMins = 1;
                        countdownTimerSecs = 10;
                        countdownTimerTicks = 0;
                        playtexture = gameTexture;
                        state = GameState.GameActive;
                    }

                    if (pad2.Buttons.A == ButtonState.Pressed)
                    {
                        countdownTimerMins = 5;
                        countdownTimerSecs = 0;
                        countdownTimerTicks = 0;
                        playtexture = gameTexture;
                        state = GameState.GameActive;
                    }
                    if (pad2.Buttons.X == ButtonState.Pressed)
                    {
                        countdownTimerMins = 4;
                        countdownTimerSecs = 0;
                        countdownTimerTicks = 0;
                        playtexture = gameTexture;
                        state = GameState.GameActive;
                    }
                    if (pad2.Buttons.Y == ButtonState.Pressed)
                    {
                        countdownTimerMins = 3;
                        countdownTimerSecs = 0;
                        countdownTimerTicks = 0;
                        playtexture = gameTexture;
                        state = GameState.GameActive;
                    }
                    if (pad2.Buttons.B == ButtonState.Pressed)
                    {
                        countdownTimerMins = 2;
                        countdownTimerSecs = 0;
                        countdownTimerTicks = 0;
                        playtexture = gameTexture;
                        state = GameState.GameActive;
                    }
                    break;
                case GameState.GameActive:
                    if (pad1.Buttons.Y == ButtonState.Pressed)
                    {
                        win.Play();
                        GamePad.SetVibration(PlayerIndex.One, 0, 0);
                        state = GameState.GameWon;
                        break;
                    }
                    countdownTimerTicks = countdownTimerTicks - 1;

                    if (countdownTimerMins == 0)
                    {
                        if (countdownTimerTicks < 30)
                        {
                            GamePad.SetVibration(PlayerIndex.One, 0, 0);
                            playtexture = gameTexture;
                        }
                        else
                        {
                            GamePad.SetVibration(PlayerIndex.One,
                                (60 - countdownTimerSecs) / 60f,
                                (60 - countdownTimerSecs) / 60f);
                            playtexture = warningTexture;
                        }
                    }

                    if (countdownTimerTicks < 0)
                    {
                        // A second has gone by
                        // reset the tick counter
                        countdownTimerTicks = 59;

                        // Play a sound
                        // Use a different sound in the last minute
                        if (countdownTimerMins > 0)
                        {
                            tick.Play();
                        }
                        else
                        {
                            loudTick.Play();
                        }

                        countdownTimerSecs = countdownTimerSecs - 1;

                        if (countdownTimerSecs < 0)
                        {
                            // A minute has gone by
                            countdownTimerSecs = 59;
                            countdownTimerMins = countdownTimerMins - 1;
                            if (countdownTimerMins < 0)
                            {
                                // We have run out of time
                                lose.Play();
                                GamePad.SetVibration(PlayerIndex.One, 0, 0);
                                state = GameState.GameLost;
                            }
                        }
                    }
                    break;
                case GameState.GameWon:
                    if (pad1.Buttons.Start == ButtonState.Pressed)
                    {
                        state = GameState.InstructionScreen;
                    }
                    break;
                case GameState.GameLost:
                    if (pad1.Buttons.Start == ButtonState.Pressed)
                    {
                        state = GameState.InstructionScreen;
                    }
                    break;
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            string leadingZero = "";

            spriteBatch.Begin();

            switch (state)
            {
                case GameState.TitleScreen:
                    spriteBatch.Draw(titleTexture, screenRectangle, Color.White);
                    break;
                case GameState.InstructionScreen:
                    spriteBatch.Draw(instructionTexture, screenRectangle, Color.White);
                    break;
                case GameState.GameActive:
                    spriteBatch.Draw(playtexture, screenRectangle, Color.White);
                    if (countdownTimerSecs < 10) leadingZero = "0";
                    drawText(countdownTimerMins.ToString() + ":" +
                        leadingZero +
                        countdownTimerSecs.ToString() + ":" +
                        (countdownTimerTicks / 6).ToString(),
                        Color.Blue, 100, 100);
                    break;
                case GameState.GameWon:
                    spriteBatch.Draw(gameWonTexture, screenRectangle, Color.White);
                    if (countdownTimerSecs < 10) leadingZero = "0";
                    drawText(countdownTimerMins.ToString() + ":" +
                        leadingZero +
                        countdownTimerSecs.ToString() + ":" +
                        (countdownTimerTicks / 6).ToString(),
                        Color.Blue, 220, 200);
                    break;
                case GameState.GameLost:
                    spriteBatch.Draw(gameLostTexture, screenRectangle, Color.White);
                    break;
            }

            spriteBatch.End();

            base.Draw(gameTime);
        }

        /// <summary>
        /// Draws text on the screen
        /// </summary>
        /// <param name="text">text to write</param>
        /// <param name="textColor">color of text</param>
        /// <param name="x">left hand edge of text</param>
        /// <param name="y">top of text</param>
        void drawText(string text, Color textColor, float x, float y)
        {
            int layer;
            Vector2 textVector = new Vector2(x, y);

            // Draw the shadow
            Color backColor = new Color(0, 0, 0, 20);
            for (layer = 0; layer < 10; layer++)
            {
                spriteBatch.DrawString(messageFont, text, textVector, backColor);
                textVector.X++;
                textVector.Y++;
            }

            // Draw the solid part of the characters
            backColor = new Color(190, 190, 190);
            for (layer = 0; layer < 5; layer++)
            {
                spriteBatch.DrawString(messageFont, text, textVector, backColor);
                textVector.X++;
                textVector.Y++;
            }

            // Draw the top of the characters
            spriteBatch.DrawString(messageFont, text, textVector, textColor);
        }

        #region Display dimension values

        // Display settings
        int rawDisplayWidth;
        int rawDisplayHeight;
        int minDisplayX;
        int maxDisplayX;
        int minDisplayY;
        int maxDisplayY;
        int displayWidth;
        int displayHeight;

        float overScanPercentage = 5.0f;

        /// <summary>
        /// Calculates percentages
        /// </summary>
        /// <param name="percentage">the percentage to be calculated</param>
        /// <param name="inputValue">the value to be worked on</param>
        /// <returns>the resulting value</returns>
        float getPercentage(float percentage, float inputValue)
        {
            return (inputValue * percentage) / 100;
        }

        private void setScreenSizeInfo()
        {
            rawDisplayWidth = graphics.GraphicsDevice.Viewport.Width;
            rawDisplayHeight = graphics.GraphicsDevice.Viewport.Height;

            float xOverscanMargin = getPercentage(overScanPercentage, rawDisplayWidth) / 2.0f;
            float yOverscanMargin = getPercentage(overScanPercentage, rawDisplayHeight) / 2.0f;

            minDisplayX = (int)(xOverscanMargin + 0.5f);
            minDisplayY = (int)(yOverscanMargin + 0.5f);

            maxDisplayX = rawDisplayWidth - minDisplayX;
            maxDisplayY = rawDisplayHeight - minDisplayY;

            displayWidth = maxDisplayX - minDisplayX;
            displayHeight = maxDisplayY - minDisplayY;
        }


        #endregion

    }
}
